@@ -19,17 +19,24 @@
 
 """A backend for the Ubuntu One Control Panel."""
 
+import operator
+import os
+
 from collections import defaultdict
 from functools import wraps
 
 from twisted.internet.defer import inlineCallbacks, returnValue
+# No name 'is_link' in module 'ubuntuone.platform'
+# pylint: disable=E0611, F0401
+from ubuntuone.platform import is_link
+from ubuntuone.platform.credentials import CredentialsManagementTool
+# pylint: enable=E0611, F0401
 
-from ubuntuone.controlpanel import dbus_client
-from ubuntuone.controlpanel import replication_client
+from ubuntuone.controlpanel import sd_client, replication_client
 from ubuntuone.controlpanel.logger import setup_logging, log_call
 # pylint: disable=W0611
-from ubuntuone.controlpanel.webclient import (UnauthorizedError,
-    WebClient, WebClientError)
+from ubuntuone.controlpanel.web_client import (UnauthorizedError,
+    web_client_factory, WebClientError)
 # pylint: enable=W0611
 
 logger = setup_logging('backend')
@@ -40,6 +47,11 @@ DEVICES_API = "1.0/devices/"
 DEVICE_REMOVE_API = "1.0/devices/remove/%s/%s"
 DEVICE_TYPE_PHONE = "Phone"
 DEVICE_TYPE_COMPUTER = "Computer"
+AUTOCONNECT_KEY = 'autoconnect'
+SHOW_ALL_NOTIFICATIONS_KEY = 'show_all_notifications'
+SHARE_AUTOSUBSCRIBE_KEY = 'share_autosubscribe'
+UDF_AUTOSUBSCRIBE_KEY = 'udf_autosubscribe'
+LIMIT_BW_KEY = 'limit_bandwidth'
 UPLOAD_KEY = "max_upload_speed"
 DOWNLOAD_KEY = "max_download_speed"
 
@@ -55,13 +67,14 @@ FILE_SYNC_UNKNOWN = 'file-sync-unknown'
 MSG_KEY = 'message'
 STATUS_KEY = 'status'
 
-BOOKMARKS_PKG = 'xul-ext-bindwood'
-CONTACTS_PKG = 'evolution-couchdb'
+CONTACTS_PKG = 'thunderbird-couchdb'
 
 
-def bool_str(value):
-    """Return the string representation of a bool (dbus-compatible)."""
-    return 'True' if value else ''
+def append_path_sep(path):
+    """If 'path' does not end with the path separator, append it."""
+    if not path.endswith(os.path.sep):
+        path += os.path.sep
+    return path
 
 
 def filter_field(info, field):
@@ -75,17 +88,17 @@ def filter_field(info, field):
 
 
 def process_unauthorized(f):
-    """Decorator to catch UnauthorizedError from the webclient and act upon."""
+    """Catch UnauthorizedError from the web_client and act upon."""
 
     @inlineCallbacks
     @wraps(f)
-    def inner(*args, **kwargs):
+    def inner(instance, *args, **kwargs):
         """Handle UnauthorizedError and clear credentials."""
         try:
-            result = yield f(*args, **kwargs)
+            result = yield f(instance, *args, **kwargs)
         except UnauthorizedError, e:
             logger.exception('process_unauthorized (clearing credentials):')
-            yield dbus_client.clear_credentials()
+            yield instance.clear_credentials()
             raise e
 
         returnValue(result)
@@ -102,16 +115,30 @@ class ControlBackend(object):
     NAME_NOT_SET = u'ENAMENOTSET'
     FREE_BYTES_NOT_AVAILABLE = u'EFREEBYTESNOTAVAILABLE'
     STATUS_DISABLED = {MSG_KEY: '', STATUS_KEY: FILE_SYNC_DISABLED}
+    DEFAULT_FILE_SYNC_SETTINGS = {
+        AUTOCONNECT_KEY: True,
+        SHOW_ALL_NOTIFICATIONS_KEY: True,
+        SHARE_AUTOSUBSCRIBE_KEY: False,
+        UDF_AUTOSUBSCRIBE_KEY: False,
+        DOWNLOAD_KEY: -1,  # no limit
+        UPLOAD_KEY: -1,  # no limit
+    }
 
     def __init__(self, shutdown_func=None):
-        """Initialize the webclient."""
-        self.shutdown_func = shutdown_func
-        self.wc = WebClient(dbus_client.get_credentials)
+        """Initialize the web_client."""
         self._status_changed_handler = None
-
+        self._credentials = None
         self._volumes = {}  # cache last known volume info
+
+        self.shutdown_func = shutdown_func
         self.file_sync_disabled = False
 
+        self.login_client = CredentialsManagementTool()
+        self.sd_client = sd_client.SyncDaemonClient()
+        self.wc = web_client_factory(self.get_credentials)
+
+        logger.info('ControlBackend: instance started.')
+
     def _process_file_sync_status(self, status):
         """Process raw file sync status into custom format.
 
@@ -123,6 +150,7 @@ class ControlBackend(object):
         * MSG_KEY: a non translatable but human readable string of the status.
 
         """
+        logger.debug('sync status: %r', status)
         if not status:
             self.file_sync_disabled = True
             return self.STATUS_DISABLED
@@ -171,8 +199,8 @@ class ControlBackend(object):
             result = self._process_file_sync_status(status)
             handler(result)
 
-        self._status_changed_handler = process_and_callback
-        dbus_client.set_status_changed_handler(process_and_callback)
+        self._status_changed_handler = handler
+        self.sd_client.set_status_changed_handler(process_and_callback)
 
     def _get_status_changed_handler(self):
         """Return the handler to be called when file sync status changes."""
@@ -182,61 +210,120 @@ class ControlBackend(object):
                                       _set_status_changed_handler)
 
     @inlineCallbacks
-    def _process_device_web_info(self, devices, enabled, limit_bw, limits,
-                                 show_notifs):
-        """Return a lis of processed devices."""
+    def _process_device_web_info(self, devices,
+        enabled=None, limit_bw=None, limits=None, autoconnect=None,
+        show_notifs=None, share_autosubscribe=None, udf_autosubscribe=None):
+        """Return a lis of processed devices.
+
+        If all the file sync settings are None, do not attach that info.
+
+        """
         result = []
         for d in devices:
             di = {}
             di["type"] = d["kind"]
-            di["name"] = d["description"]
-            di["configurable"] = ''
+            di["name"] = d["description"] if d["description"] \
+                                          else self.NAME_NOT_SET
             if di["type"] == DEVICE_TYPE_COMPUTER:
                 di["device_id"] = di["type"] + d["token"]
             if di["type"] == DEVICE_TYPE_PHONE:
                 di["device_id"] = di["type"] + str(d["id"])
 
             is_local = yield self.device_is_local(di["device_id"])
-            di["is_local"] = bool_str(is_local)
-            # currently, only local devices are configurable.
-            # eventually, more devices will be configurable.
-            di["configurable"] = bool_str(is_local and enabled)
-
-            if bool(di["configurable"]):
-                di["limit_bandwidth"] = bool_str(limit_bw)
-                di["show_all_notifications"] = bool_str(show_notifs)
-                upload = limits["upload"]
-                download = limits["download"]
-                di[UPLOAD_KEY] = str(upload)
-                di[DOWNLOAD_KEY] = str(download)
-
-            # date_added is not in the webservice yet (LP: #673668)
-            # di["date_added"] = ""
-
-            # missing values (LP: #673668)
-            # di["available_services"] = ""
-            # di["enabled_services"] = ""
-
-            # make a sanity check
-            for key, val in di.iteritems():
-                if not isinstance(val, basestring):
-                    logger.warning('_process_device_web_info: (key %r), '
-                                   'val %r is not a basestring.', key, val)
-                    di[key] = repr(val)
+            di["is_local"] = is_local
 
             if is_local:  # prepend the local device!
                 result.insert(0, di)
             else:
                 result.append(di)
 
+            if enabled is None:
+                # without knowing if file sync is enabled or not,
+                # we can't add any extra info about the device.
+                continue
+
+            # currently, only local devices are configurable.
+            di["configurable"] = is_local and enabled
+
+            if di["configurable"]:
+                di[LIMIT_BW_KEY] = limit_bw
+                di[AUTOCONNECT_KEY] = autoconnect
+                di[SHOW_ALL_NOTIFICATIONS_KEY] = show_notifs
+                di[SHARE_AUTOSUBSCRIBE_KEY] = share_autosubscribe
+                di[UDF_AUTOSUBSCRIBE_KEY] = udf_autosubscribe
+                di[UPLOAD_KEY] = limits["upload"]
+                di[DOWNLOAD_KEY] = limits["download"]
+
         returnValue(result)
 
+    @inlineCallbacks
+    def _process_device_local_info(self,
+        enabled=None, limit_bw=None, limits=None, autoconnect=None,
+        show_notifs=None, share_autosubscribe=None, udf_autosubscribe=None):
+        """Return the information for the local device.
+
+        If all the file sync settings are None, do not attach that info.
+
+        """
+        credentials = yield self.get_credentials()
+
+        local_device = {}
+        local_device["type"] = DEVICE_TYPE_COMPUTER
+        local_device["name"] = credentials['name']
+        device_id = local_device["type"] + credentials["token"]
+        local_device["device_id"] = device_id
+        local_device["is_local"] = True
+
+        if enabled is not None:
+            local_device["configurable"] = enabled
+            if local_device["configurable"]:
+                local_device[LIMIT_BW_KEY] = limit_bw
+                local_device[AUTOCONNECT_KEY] = autoconnect
+                local_device[SHOW_ALL_NOTIFICATIONS_KEY] = show_notifs
+                local_device[SHARE_AUTOSUBSCRIBE_KEY] = share_autosubscribe
+                local_device[UDF_AUTOSUBSCRIBE_KEY] = udf_autosubscribe
+                upload = limits["upload"]
+                download = limits["download"]
+                local_device[UPLOAD_KEY] = upload
+                local_device[DOWNLOAD_KEY] = download
+
+        returnValue(local_device)
+
+    def _process_path(self, path):
+        """Trim 'path' so the '~' is removed."""
+        home = os.path.expanduser('~')
+        result = path.replace(os.path.join(home, ''), '')
+        return result
+
+    @inlineCallbacks
+    def get_credentials(self):
+        """Find credentials."""
+        if not self._credentials:
+            self._credentials = yield self.login_client.find_credentials()
+        returnValue(self._credentials)
+
+    @inlineCallbacks
+    def clear_credentials(self):
+        """Clear the credentials."""
+        self._credentials = None
+        yield self.login_client.clear_credentials()
+
     @inlineCallbacks
     def get_token(self):
         """Return the token from the credentials."""
-        credentials = yield dbus_client.get_credentials()
+        credentials = yield self.get_credentials()
         returnValue(credentials["token"])
 
+    @log_call(logger.debug, with_args=False)
+    @inlineCallbacks
+    def login(self, email, password):
+        """Login using 'email' and 'password'."""
+        result = yield self.login_client.login_email_password(
+                    email=email, password=password)
+        # cache credentils
+        self._credentials = result
+        returnValue(result)
+
     @inlineCallbacks
     def device_is_local(self, device_id):
         """Return whether 'device_id' is the local devicew or not."""
@@ -268,8 +355,8 @@ class ControlBackend(object):
         result["email"] = account_info["email"]
 
         quota_info = yield self.wc.call_api(QUOTA_API)
-        result["quota_total"] = str(quota_info["total"])
-        result["quota_used"] = str(quota_info["used"])
+        result["quota_total"] = quota_info["total"]
+        result["quota_used"] = quota_info["used"]
 
         returnValue(result)
 
@@ -278,16 +365,35 @@ class ControlBackend(object):
     @inlineCallbacks
     def devices_info(self):
         """Get the user devices info."""
-        result = limit_bw = limits = show_notifs = None
-        enabled = yield dbus_client.files_sync_enabled()
+        result = limit_bw = limits = None
+        autoconnect = show_notifs = None
+        share_autosubscribe = udf_autosubscribe = None
+
+        enabled = yield self.sd_client.files_sync_enabled()
+        enabled = bool(enabled)
         if enabled:
-            limit_bw = yield dbus_client.bandwidth_throttling_enabled()
-            show_notifs = yield dbus_client.show_all_notifications_enabled()
-            limits = yield dbus_client.get_throttling_limits()
+            sd_res = yield self.sd_client.autoconnect_enabled()
+            autoconnect = bool(sd_res)
+
+            sd_res = yield self.sd_client.show_all_notifications_enabled()
+            show_notifs = bool(sd_res)
+
+            sd_res = yield self.sd_client.share_autosubscribe_enabled()
+            share_autosubscribe = bool(sd_res)
+
+            sd_res = yield self.sd_client.udf_autosubscribe_enabled()
+            udf_autosubscribe = bool(sd_res)
+
+            sd_res = yield self.sd_client.bandwidth_throttling_enabled()
+            limit_bw = bool(sd_res)
+
+            limits = yield self.sd_client.get_throttling_limits()
 
         logger.debug('devices_info: file sync enabled? %s limit_bw %s, limits '
-                     '%s, show_notifs %s',
-                     enabled, limit_bw, limits, show_notifs)
+                     '%s, autoconnect %s, show_notifs %s, '
+                     'share_autosubscribe %s, udf_autosubscribe %s',
+                     enabled, limit_bw, limits, autoconnect, show_notifs,
+                     share_autosubscribe, udf_autosubscribe)
 
         try:
             devices = yield self.wc.call_api(DEVICES_API)
@@ -297,32 +403,41 @@ class ControlBackend(object):
             logger.exception('devices_info: web client failure:')
         else:
             result = yield self._process_device_web_info(devices, enabled,
-                                                         limit_bw, limits,
-                                                         show_notifs)
+                        limit_bw, limits, autoconnect, show_notifs,
+                        share_autosubscribe, udf_autosubscribe)
+
         if result is None:
-            logger.info('devices_info: result is None after calling '
-                        'devices/ API, building the local device.')
-            credentials = yield dbus_client.get_credentials()
-            local_device = {}
-            local_device["type"] = DEVICE_TYPE_COMPUTER
-            local_device["name"] = credentials['name']
-            device_id = local_device["type"] + credentials["token"]
-            local_device["device_id"] = device_id
-            local_device["is_local"] = bool_str(True)
-            local_device["configurable"] = bool_str(enabled)
-            if bool(local_device["configurable"]):
-                local_device["limit_bandwidth"] = bool_str(limit_bw)
-                show_notifs = bool_str(show_notifs)
-                local_device["show_all_notifications"] = show_notifs
-                upload = limits["upload"]
-                download = limits["download"]
-                local_device[UPLOAD_KEY] = str(upload)
-                local_device[DOWNLOAD_KEY] = str(download)
+            local_device = yield self._process_device_local_info(enabled,
+                            limit_bw, limits, autoconnect, show_notifs,
+                            share_autosubscribe, udf_autosubscribe)
             result = [local_device]
+
+        logger.info('devices_info: result is %r',
+                    filter_field(result, field='device_id'))
+
+        returnValue(result)
+
+    @log_call(logger.debug)
+    @process_unauthorized
+    @inlineCallbacks
+    def device_names_info(self):
+        """Get the user devices info, only list names and kind."""
+        result = None
+        try:
+            devices = yield self.wc.call_api(DEVICES_API)
+        except UnauthorizedError:
+            raise
+        except WebClientError:
+            logger.exception('device_names_info: web client failure:')
         else:
-            logger.info('devices_info: result is not None after calling '
-                        'devices/ API: %r',
-                        filter_field(result, field='device_id'))
+            result = yield self._process_device_web_info(devices)
+
+        if result is None:
+            local_device = yield self._process_device_local_info()
+            result = [local_device]
+
+        logger.info('device_names_info: result is %r',
+                    filter_field(result, field='device_id'))
 
         returnValue(result)
 
@@ -340,23 +455,21 @@ class ControlBackend(object):
         """Change the settings for the given device."""
         is_local = yield self.device_is_local(device_id)
 
-        if is_local and "show_all_notifications" in settings:
-            show_all_notif = bool(settings["show_all_notifications"])
-            if not show_all_notif:
-                yield dbus_client.disable_show_all_notifications()
+        if is_local and SHOW_ALL_NOTIFICATIONS_KEY in settings:
+            if not settings[SHOW_ALL_NOTIFICATIONS_KEY]:
+                yield self.sd_client.disable_show_all_notifications()
             else:
-                yield dbus_client.enable_show_all_notifications()
+                yield self.sd_client.enable_show_all_notifications()
 
-        if is_local and "limit_bandwidth" in settings:
-            limit_bw = bool(settings["limit_bandwidth"])
-            if not limit_bw:
-                yield dbus_client.disable_bandwidth_throttling()
+        if is_local and LIMIT_BW_KEY in settings:
+            if not settings[LIMIT_BW_KEY]:
+                yield self.sd_client.disable_bandwidth_throttling()
             else:
-                yield dbus_client.enable_bandwidth_throttling()
+                yield self.sd_client.enable_bandwidth_throttling()
 
         if is_local and (UPLOAD_KEY in settings or
                          DOWNLOAD_KEY in settings):
-            current_limits = yield dbus_client.get_throttling_limits()
+            current_limits = yield self.sd_client.get_throttling_limits()
             limits = {
                 "download": current_limits["download"],
                 "upload": current_limits["upload"],
@@ -365,7 +478,7 @@ class ControlBackend(object):
                 limits["upload"] = settings[UPLOAD_KEY]
             if DOWNLOAD_KEY in settings:
                 limits["download"] = settings[DOWNLOAD_KEY]
-            dbus_client.set_throttling_limits(limits)
+            self.sd_client.set_throttling_limits(limits)
 
         # still pending: more work on the settings dict (LP: #673674)
         returnValue(device_id)
@@ -384,7 +497,7 @@ class ControlBackend(object):
         if is_local:
             logger.warning('remove_device: device is local! removing and '
                            'clearing credentials.')
-            yield dbus_client.clear_credentials()
+            yield self.clear_credentials()
 
         returnValue(device_id)
 
@@ -392,9 +505,9 @@ class ControlBackend(object):
     @inlineCallbacks
     def file_sync_status(self):
         """Return the status of the file sync service."""
-        enabled = yield dbus_client.files_sync_enabled()
+        enabled = yield self.sd_client.files_sync_enabled()
         if enabled:
-            status = yield dbus_client.get_current_status()
+            status = yield self.sd_client.get_current_status()
         else:
             status = {}
         returnValue(self._process_file_sync_status(status))
@@ -403,70 +516,71 @@ class ControlBackend(object):
     @inlineCallbacks
     def enable_files(self):
         """Enable the files service."""
-        yield dbus_client.set_files_sync_enabled(True)
+        yield self.sd_client.set_files_sync_enabled(True)
         self.file_sync_disabled = False
 
     @log_call(logger.debug)
     @inlineCallbacks
     def disable_files(self):
         """Enable the files service."""
-        yield dbus_client.set_files_sync_enabled(False)
+        yield self.sd_client.set_files_sync_enabled(False)
         self.file_sync_disabled = True
 
     @log_call(logger.debug)
     @inlineCallbacks
     def connect_files(self):
         """Connect the files service."""
-        yield dbus_client.connect_file_sync()
+        yield self.sd_client.connect_file_sync()
 
     @log_call(logger.debug)
     @inlineCallbacks
     def disconnect_files(self):
         """Disconnect the files service."""
-        yield dbus_client.disconnect_file_sync()
+        yield self.sd_client.disconnect_file_sync()
 
     @log_call(logger.debug)
     @inlineCallbacks
     def restart_files(self):
         """restart the files service."""
-        yield dbus_client.stop_file_sync()
-        yield dbus_client.start_file_sync()
+        yield self.sd_client.stop_file_sync()
+        yield self.sd_client.start_file_sync()
 
     @log_call(logger.debug)
     @inlineCallbacks
     def start_files(self):
         """start the files service."""
-        yield dbus_client.start_file_sync()
+        yield self.sd_client.start_file_sync()
 
     @log_call(logger.debug)
     @inlineCallbacks
     def stop_files(self):
         """stop the files service."""
-        yield dbus_client.stop_file_sync()
+        yield self.sd_client.stop_file_sync()
 
     @log_call(logger.debug)
     @inlineCallbacks
-    def volumes_info(self):
+    def volumes_info(self, with_storage_info=True):
         """Get the volumes info."""
         self._volumes = {}
 
-        try:
-            account = yield self.account_info()
-        except Exception:  # pylint: disable=W0703
-            logger.exception('volumes_info: quota could not be retrieved:')
-            free_bytes = self.FREE_BYTES_NOT_AVAILABLE
-        else:
-            free_bytes = int(account['quota_total']) - \
-                         int(account['quota_used'])
+        free_bytes = self.FREE_BYTES_NOT_AVAILABLE
+        if with_storage_info:
+            try:
+                account = yield self.account_info()
+            except Exception:  # pylint: disable=W0703
+                logger.exception('volumes_info: quota could not be retrieved:')
+            else:
+                free_bytes = account['quota_total'] - account['quota_used']
 
-        root_dir = yield dbus_client.get_root_dir()
-        shares_dir = yield dbus_client.get_shares_dir()
-        shares_dir_link = yield dbus_client.get_shares_dir_link()
-        folders = yield dbus_client.get_folders()
-        shares = yield dbus_client.get_shares()
+        root_dir = yield self.sd_client.get_root_dir()
+        shares_dir = yield self.sd_client.get_shares_dir()
+        shares_dir_link = yield self.sd_client.get_shares_dir_link()
+        folders = yield self.sd_client.get_folders()
+        shares = yield self.sd_client.get_shares()
 
         root_volume = {u'volume_id': u'', u'path': root_dir,
-                       u'subscribed': 'True', u'type': self.ROOT_TYPE}
+                       u'subscribed': True, u'type': self.ROOT_TYPE,
+                       u'display_name': self._process_path(root_dir)}
         self._volumes[u''] = root_volume
 
         # group shares by the offering user
@@ -483,8 +597,11 @@ class ControlBackend(object):
                                'list (%r).', vid, self._volumes[vid])
             self._volumes[vid] = share
 
+            share[u'realpath'] = share[u'path']
             nicer_path = share[u'path'].replace(shares_dir, shares_dir_link)
             share[u'path'] = nicer_path
+            share[u'subscribed'] = bool(share[u'subscribed'])
+            share[u'display_name'] = share[u'name']
 
             username = share['other_visible_name']
             if not username:
@@ -500,16 +617,20 @@ class ControlBackend(object):
             if vid in self._volumes:
                 logger.warning('volumes_info: udf %r already in the volumes '
                                'list (%r).', vid, self._volumes[vid])
+            folder[u'subscribed'] = bool(folder[u'subscribed'])
+            folder[u'display_name'] = self._process_path(folder[u'path'])
             self._volumes[vid] = folder
 
-        result = [(u'', unicode(free_bytes), [root_volume] + folders)]
+        folders.sort(key=operator.itemgetter('path'))
+        result = [(u'', free_bytes, [root_volume] + folders)]
 
         for other_username, shares in shares_result.iteritems():
             send_freebytes = any(s['access_level'] == 'Modify' for s in shares)
             if send_freebytes:
-                free_bytes = shares[0][u'free_bytes']
+                free_bytes = int(shares[0][u'free_bytes'])
             else:
                 free_bytes = self.FREE_BYTES_NOT_AVAILABLE
+            shares.sort(key=operator.itemgetter('path'))
             result.append((other_username, free_bytes, shares))
 
         returnValue(result)
@@ -523,27 +644,59 @@ class ControlBackend(object):
 
         """
         if 'subscribed' in settings:
-            subscribed = bool(settings['subscribed'])
+            subscribed = settings['subscribed']
             if subscribed:
                 yield self.subscribe_volume(volume_id)
             else:
                 yield self.unsubscribe_volume(volume_id)
 
+        returnValue(volume_id)
+
     @inlineCallbacks
     def subscribe_volume(self, volume_id):
         """Subscribe to 'volume_id'."""
         if self._volumes[volume_id][u'type'] == self.FOLDER_TYPE:
-            yield dbus_client.subscribe_folder(volume_id)
+            yield self.sd_client.subscribe_folder(volume_id)
         elif self._volumes[volume_id][u'type'] == self.SHARE_TYPE:
-            yield dbus_client.subscribe_share(volume_id)
+            yield self.sd_client.subscribe_share(volume_id)
 
     @inlineCallbacks
     def unsubscribe_volume(self, volume_id):
         """Unsubscribe from 'volume_id'."""
         if self._volumes[volume_id][u'type'] == self.FOLDER_TYPE:
-            yield dbus_client.unsubscribe_folder(volume_id)
+            yield self.sd_client.unsubscribe_folder(volume_id)
         elif self._volumes[volume_id][u'type'] == self.SHARE_TYPE:
-            yield dbus_client.unsubscribe_share(volume_id)
+            yield self.sd_client.unsubscribe_share(volume_id)
+
+    @log_call(logger.debug)
+    @inlineCallbacks
+    def create_folder(self, folder_path):
+        """Create a new User Defined Folder pointing to 'folder_path'."""
+        yield self.sd_client.create_folder(path=folder_path)
+
+    @log_call(logger.debug)
+    @inlineCallbacks
+    def validate_path_for_folder(self, folder_path):
+        """Validate 'folder_path' for folder creation."""
+        user_home = os.path.expanduser('~')
+        folder_path = append_path_sep(folder_path)
+
+        # handle folder_path not within '~' or links
+        # XXX is_link expects bytes, see bug #824252
+        if not folder_path.startswith(user_home) or is_link(
+            folder_path.encode('utf-8')):
+            returnValue(False)
+
+        # handle folder_path nested with a existing cloud folder
+        volumes = yield self.volumes_info(with_storage_info=False)
+        for _, _, data in volumes:
+            for volume in data:
+                cloud_folder = append_path_sep(volume['path'])
+                if (folder_path.startswith(cloud_folder) or
+                    cloud_folder.startswith(folder_path)):
+                    returnValue(False)
+
+        returnValue(True)
 
     @log_call(logger.debug)
     @inlineCallbacks
@@ -555,15 +708,13 @@ class ControlBackend(object):
         result = []
         for rep in replications:
             dependency = ''
-            if rep == replication_client.BOOKMARKS:
-                dependency = BOOKMARKS_PKG
-            elif rep == replication_client.CONTACTS:
+            if rep == replication_client.CONTACTS:
                 dependency = CONTACTS_PKG
 
             repd = {
                 "replication_id": rep,
                 "name": rep,  # this may change to be more user friendly
-                "enabled": bool_str(rep not in exclusions),
+                "enabled": rep not in exclusions,
                 "dependency": dependency,
             }
             result.append(repd)
@@ -575,23 +726,70 @@ class ControlBackend(object):
     def change_replication_settings(self, replication_id, settings):
         """Change the settings for the given replication."""
         if 'enabled' in settings:
-            if bool(settings['enabled']):
+            if settings['enabled']:
                 yield replication_client.replicate(replication_id)
             else:
                 yield replication_client.exclude(replication_id)
         returnValue(replication_id)
 
     @log_call(logger.debug)
-    def query_bookmark_extension(self):
-        """True if the bookmark extension has been installed."""
-        # still pending (LP: #673672)
-        returnValue(False)
+    @inlineCallbacks
+    def file_sync_settings_info(self):
+        """Get the file sync settings info."""
+        result = {}
 
-    @log_call(logger.debug)
-    def install_bookmarks_extension(self):
-        """Install the extension to sync bookmarks."""
-        # still pending (LP: #673673)
-        returnValue(None)
+        for name in (AUTOCONNECT_KEY, SHOW_ALL_NOTIFICATIONS_KEY,
+                     SHARE_AUTOSUBSCRIBE_KEY, UDF_AUTOSUBSCRIBE_KEY):
+            sd_method = getattr(self.sd_client, '%s_enabled' % name)
+            value = yield sd_method()
+            result[name] = bool(value)
+
+        limits = yield self.sd_client.get_throttling_limits()
+        result[DOWNLOAD_KEY] = limits['download']
+        result[UPLOAD_KEY] = limits['upload']
+
+        returnValue(result)
+
+    @inlineCallbacks
+    def _change_boolean_file_sync_setting(self, setting_name, settings):
+        """Change the value for 'setting_name' to be 'new_value'."""
+        if setting_name in settings:
+            new_value = settings[setting_name]
+            sd_method_name = 'enable_%s' if new_value else 'disable_%s'
+            sd_method = getattr(self.sd_client, sd_method_name % setting_name)
+            yield sd_method()
+
+    @log_call(logger.info)
+    @inlineCallbacks
+    def change_file_sync_settings(self, settings):
+        """Change the file sync settings."""
+        for name in (AUTOCONNECT_KEY, SHOW_ALL_NOTIFICATIONS_KEY,
+                     SHARE_AUTOSUBSCRIBE_KEY, UDF_AUTOSUBSCRIBE_KEY):
+            yield self._change_boolean_file_sync_setting(name, settings)
+
+        if DOWNLOAD_KEY in settings or UPLOAD_KEY in settings:
+            current_limits = yield self.sd_client.get_throttling_limits()
+            limits = {
+                "download": current_limits["download"],
+                "upload": current_limits["upload"],
+            }
+            if UPLOAD_KEY in settings:
+                limits["upload"] = settings[UPLOAD_KEY]
+            if DOWNLOAD_KEY in settings:
+                limits["download"] = settings[DOWNLOAD_KEY]
+            yield self.sd_client.set_throttling_limits(limits)
+
+            throttling_disabled = sum(limits.itervalues()) == -2
+            if throttling_disabled:
+                yield self.sd_client.disable_bandwidth_throttling()
+            else:
+                yield self.sd_client.enable_bandwidth_throttling()
+
+    @log_call(logger.info)
+    @inlineCallbacks
+    def restore_file_sync_settings(self):
+        """Restore the file sync settings."""
+        yield self.change_file_sync_settings(self.DEFAULT_FILE_SYNC_SETTINGS)
 
     @log_call(logger.info)
     def shutdown(self):
