@@ -20,14 +20,23 @@
 """Export the control backend thru DBus."""
 
 from functools import wraps
+import sys
 
 import dbus.service
-import gobject
+# pylint: disable=E0611
+# pylint: disable=W0404
+if 'gobject' in sys.modules:
+    import gobject as GObject
+else:
+    from gi.repository import GObject
+# pylint: enable=W0404
+# pylint: enable=E0611
 
 from dbus.mainloop.glib import DBusGMainLoop
 from dbus.service import method, signal
 
 from twisted.python.failure import Failure
+from ubuntuone.syncdaemon.interaction_interfaces import bool_str
 
 from ubuntuone.controlpanel import (DBUS_BUS_NAME, DBUS_PREFERENCES_PATH,
     DBUS_PREFERENCES_IFACE)
@@ -48,12 +57,45 @@ logger = setup_logging('dbus_service')
 
 def make_unicode(anything):
     """Transform 'anything' on an unicode."""
-    if not isinstance(anything, unicode):
+    if isinstance(anything, bool):
+        anything = bool_str(anything)
+    elif not isinstance(anything, unicode):
         anything = str(anything).decode('utf8', 'replace')
 
     return anything
 
 
+def transform_info(f):
+    """Decorator to apply to DBus success signals.
+
+    With this call, a list of dicts with string keys and any values is
+    transformed into a list with string-string dicts.
+
+    """
+    def inner(info):
+        """Do the info transformation."""
+        logger.debug('processing info: %r', info)
+
+        def process_dict(data):
+            """Stringify dict's values."""
+            result = {}
+            for key, val in data.iteritems():
+                result[key] = make_unicode(val)
+            return result
+
+        if isinstance(info, dict):
+            result = process_dict(info)
+        else:
+            result = []
+            for data in info:
+                result.append(process_dict(data))
+
+        result = f(result)
+        return result
+
+    return inner
+
+
 def error_handler(error):
     """Handle 'error' properly to be able to call a dbus error signal.
      If 'error' is a Failure, then transform the exception in it to a error
@@ -134,7 +176,7 @@ class ControlPanelBackend(dbus.service.Object):
     def account_info(self):
         """Find out the account info for the current logged in user."""
         d = self.backend.account_info()
-        d.addCallback(self.AccountInfoReady)
+        d.addCallback(transform_info(self.AccountInfoReady))
         d.addErrback(self.transform(self.AccountInfoError))
 
     @log_call(logger.debug)
@@ -154,7 +196,7 @@ class ControlPanelBackend(dbus.service.Object):
     def devices_info(self):
         """Find out the devices info for the logged in user."""
         d = self.backend.devices_info()
-        d.addCallback(self.DevicesInfoReady)
+        d.addCallback(transform_info(self.DevicesInfoReady))
         d.addErrback(self.transform(self.DevicesInfoError))
 
     @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="aa{ss}")
@@ -430,12 +472,21 @@ class ControlPanelBackend(dbus.service.Object):
 
     #---
 
+    def process_volumes(self, info):
+        """Stringify the volumes info sent from the backend."""
+        result = []
+        f = lambda _: _
+        for name, free_bytes, data in info:
+            result.append((name, make_unicode(free_bytes),
+                           transform_info(f)(data)))
+        self.VolumesInfoReady(result)
+
     @log_call(logger.debug)
     @method(dbus_interface=DBUS_PREFERENCES_IFACE, in_signature="")
     def volumes_info(self):
         """Find out the volumes info for the logged in user."""
         d = self.backend.volumes_info()
-        d.addCallback(self.VolumesInfoReady)
+        d.addCallback(self.process_volumes)
         d.addErrback(self.transform(self.VolumesInfoError))
 
     @log_call(logger.debug)
@@ -476,7 +527,7 @@ class ControlPanelBackend(dbus.service.Object):
     def replications_info(self):
         """Return the replications info."""
         d = self.backend.replications_info()
-        d.addCallback(self.ReplicationsInfoReady)
+        d.addCallback(transform_info(self.ReplicationsInfoReady))
         d.addErrback(self.transform(self.ReplicationsInfoError))
 
     @log_call(logger.debug)
@@ -512,46 +563,6 @@ class ControlPanelBackend(dbus.service.Object):
 
     #---
 
-    @log_call(logger.debug)
-    @method(dbus_interface=DBUS_PREFERENCES_IFACE, in_signature="")
-    def query_bookmark_extension(self):
-        """Check if the extension to sync bookmarks is installed."""
-        d = self.backend.query_bookmark_extension()
-        d.addCallback(self.QueryBookmarksResult)
-        d.addErrback(self.transform(self.QueryBookmarksError))
-
-    @log_call(logger.debug)
-    @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="b")
-    def QueryBookmarksResult(self, enabled):
-        """The bookmark extension is or is not installed."""
-
-    @log_call(logger.error)
-    @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="a{ss}")
-    def QueryBookmarksError(self, error):
-        """Problem getting the status of the extension."""
-
-    #---
-
-    @log_call(logger.info)
-    @method(dbus_interface=DBUS_PREFERENCES_IFACE, in_signature="")
-    def install_bookmarks_extension(self):
-        """Install the extension to sync bookmarks."""
-        d = self.backend.install_bookmarks_extension()
-        d.addCallback(lambda _: self.InstallBookmarksSuccess())
-        d.addErrback(self.transform(self.InstallBookmarksError))
-
-    @log_call(logger.info)
-    @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="")
-    def InstallBookmarksSuccess(self):
-        """The extension to sync bookmarks has been installed."""
-
-    @log_call(logger.error)
-    @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="a{ss}")
-    def InstallBookmarksError(self, error):
-        """Problem installing the extension to sync bookmarks."""
-
-    #---
-
     @log_call(logger.info)
     @method(dbus_interface=DBUS_PREFERENCES_IFACE, in_signature="")
     def shutdown(self):
@@ -565,9 +576,9 @@ def init_mainloop():
 
 
 def run_mainloop(loop=None):
-    """Run the gobject main loop."""
+    """Run the GObject main loop."""
     if loop is None:
-        loop = gobject.MainLoop()
+        loop = GObject.MainLoop()
     loop.run()
 
 
@@ -600,7 +611,7 @@ def main():
     """Hook the DBus listeners and start the main loop."""
     init_mainloop()
     if register_service():
-        loop = gobject.MainLoop()
+        loop = GObject.MainLoop()
         publish_backend(shutdown_func=loop.quit)
         run_mainloop(loop=loop)
     else:
